@@ -770,16 +770,19 @@ pub mod tests {
         let mut lo = Lowerer::<Vec<u8>>::new(pool);
         let start = lo.here().target().unwrap();
         for (dest, src1, src2) in [
+            (RA, RA, RA),
             (RA, RA, RD),
             (RA, RA, RC),
             (RA, RC, RD),
             (RA, RC, RC),
 
+            (RD, RA, RA),
             (RD, RA, RD),
             (RD, RA, RC),
             (RD, RC, RD),
             (RD, RC, RC),
 
+            (RC, RA, RA),
             (RC, RA, RD),
             (RC, RA, RC),
             (RC, RC, RD),
@@ -788,16 +791,19 @@ pub mod tests {
             lo.shift_binary(Shl, P32, dest, src1, src2);
         }
         disassemble(&lo.a, start, vec![
+            "mov r12,rcx", "mov rcx,rax", "shl eax,cl", "mov rcx,r12",
             "mov r12,rcx", "mov rcx,rdx", "shl eax,cl", "mov rcx,r12",
             "shl eax,cl",
             "mov r12,rcx", "mov rcx,rdx", "mov rax,r12", "shl eax,cl", "mov rcx,r12",
             "mov rax,rcx", "shl eax,cl",
 
+            "mov r12,rcx", "mov rcx,rax", "mov rdx,rax", "shl edx,cl", "mov rcx,r12",
             "mov r12,rcx", "mov rcx,rdx", "mov rdx,rax", "shl edx,cl", "mov rcx,r12",
             "mov rdx,rax", "shl edx,cl",
             "mov r12,rcx", "mov rcx,rdx", "mov rdx,r12", "shl edx,cl", "mov rcx,r12",
             "mov rdx,rcx", "shl edx,cl",
 
+            "mov rcx,rax", "mov r12,rax", "shl r12d,cl", "mov rcx,r12",
             "mov rcx,rdx", "mov r12,rax", "shl r12d,cl", "mov rcx,r12",
             "mov r12,rax", "shl r12d,cl", "mov rcx,r12",
             "mov r12,rcx", "mov rcx,rdx", "shl r12d,cl", "mov rcx,r12",
